using System;
using System.Collections;

namespace Test
{
	/// <summary>
	/// Summary description for MostRecentlyUsedList.
	/// </summary>
	[Serializable]
	public class MostRecentlyUsedList : IList
	{
		private int _capacity;
		private ArrayList _list;

		/// <summary>
		/// Initializes a new instance of the MostRecentlyUsedList class 
		/// that is empty and has the specified capacity
		/// </summary>
		/// <param name="capacity">The number of elements that the new 
		/// MostRecentlyUsedList is capable of storing. </param>
		public MostRecentlyUsedList(int capacity)
		{
			_capacity = capacity;
			_list = new ArrayList(capacity);
		}

		/// <summary>
		/// Add a object to the beginning of the list. If the value already 
		/// exists it is removed first. The length of the list will never 
		/// exceed the capacity.
		/// </summary>
		/// <param name="value">The Object to be added to the beginning 
		/// of the MostRecentlyUsedList</param>
		public void Add(object value) 
		{
			if (_list.Contains(value)) {
				_list.Remove(value);
			} else {
				if (this.Count == this.Capacity) {
					_list.RemoveAt(this.Count - 1);
				}
			}
			_list.Insert(0, value);
		}

		/// <summary>
		/// Get or sets the number of elements that the MostRecentlyUsedList 
		/// is capable of storing.
		/// </summary>
		public int Capacity {
			get { return (this._capacity);	}
			set {
				this._capacity = value;
				while (this.Count >= this.Capacity) {
					_list.RemoveAt(this.Count - 1);
				}
			}
		}

		/// <summary>
		/// Remove all elements form the MostRecentlyUsedList
		/// </summary>
		public void Clear()
		{
			_list.Clear();
		}

		/// <summary>
		/// Determines whether an element is in the MostRecentlyUsedList.
		/// </summary>
		/// <param name="value">The Object to locate in the MostRecentlyUsedList. 
		/// The element to locate can be a null reference (Nothing in Visual Basic). </param>
		/// <returns>true if item is found in the MostRecentlyUsedList; otherwise, false.</returns>
		public bool Contains(object value) {
			return _list.Contains(value);
		}

		/// <summary>
		/// Gets the number of elements actually contained in the MostRecentlyUsedList.
		/// </summary>
		public int Count
		{
			get { return _list.Count; }
		}

		/// <summary>
		/// Returns an enumerator that can iterate through the MostRecentlyUsedList.
		/// </summary>
		/// <returns>An IEnumerator for the entire MostRecentlyUsedList.</returns>
		public IEnumerator GetEnumerator()
		{
			return _list.GetEnumerator();
		}

		/// <summary>
		/// Returns the zero-based index of the first occurrence of a value in the
		///  MostRecentlyUsedList or in a portion of it.
		/// </summary>
		/// <param name="value">The Object to locate in the MostRecentlyUsedList.</param>
		/// <returns>The zero-based index of the first occurrence of value within 
		/// the entire MostRecentlyUsedList, if found; otherwise, -1.
		/// </returns>
		public int IndexOf(object value) {
			return _list.IndexOf(value);
		}

		/// <summary>
		/// Gets or sets the element at the specified index. 
		/// </summary>
		public object this[int index]
		{
			get {
				return _list[index];
			}
			set
			{
				_list[index] = value;
			}
		}

		#region Interface implementations (ICollection & IList)

		void ICollection.CopyTo(Array array, int index)
		{
			_list.CopyTo(array, index);
		}

		bool ICollection.IsSynchronized
		{
			get
			{
				return _list.IsSynchronized;
			}
		}

		object ICollection.SyncRoot
		{
			get
			{
				return _list.SyncRoot;
			}
		}

		int IList.Add(object value)
		{
			this.Add(value);
			return 0;
		}

		void IList.Insert(int index, object value)
		{
			throw new InvalidOperationException("Insert is not allowed!");
		}

		bool IList.IsFixedSize
		{
			get
			{
				return _list.IsFixedSize;
			}
		}

		bool IList.IsReadOnly
		{
			get { return _list.IsReadOnly; }
		}

		void IList.Remove(object value)
		{
			throw new InvalidOperationException("Remove is not allowed!");
		}

		void IList.RemoveAt(int index)
		{
			throw new InvalidOperationException("RemoveAt is not allowed!");
		}
		#endregion

	}
}
