using System;
using System.IO;
using System.IO.IsolatedStorage;
using System.Collections;
using System.Windows.Forms;
using System.Runtime.Serialization;
using System.Runtime.Serialization.Formatters.Binary;
using System.Reflection;

namespace ReflectionIT.UserSettings
{
	/// <summary>
	/// The IsolatedUserSettings class can be used to store and retrieve user settings 
	/// in and from an IsolatedStorageFile.
	/// <remarks>
	/// Copyright  2003 Reflection IT
	/// 
	/// This software is provided 'as-is', without any express or implied warranty.
	/// In no event will the authors be held liable for any damages arising from the
	/// use of this software.
	/// 
	/// Permission is granted to anyone to use this software for any purpose,
	/// including commercial applications, subject to the following restrictions:
	/// 
	/// 1. The origin of this software must not be misrepresented; you must not claim
	/// that you wrote the original software. 
	/// 
	/// 2. No substantial portion of the source code of this library may be redistributed
	/// without the express written permission of the copyright holders, where
	/// "substantial" is defined as enough code to be recognizably from this library. 
	/// 
	/// </remarks>
	/// </summary>
	public class IsolatedUserSettings : IDisposable
	{
		private Hashtable _settings;
		private bool _closed = false;
		private string _fileName;
		private IsolatedStorageScope _scope;

		/// <summary>
		/// Constructor with IsolatedStorageScope: Assembly | User | Domain | Roaming
		/// </summary>
		public IsolatedUserSettings(string fileName) : this(fileName, IsolatedStorageScope.Assembly | IsolatedStorageScope.User | IsolatedStorageScope.Domain | IsolatedStorageScope.Roaming) {
		}

		/// <summary>
		/// Constructor
		/// </summary>
		public IsolatedUserSettings(string fileName, IsolatedStorageScope scope)
		{
			_scope = scope;
			_fileName = fileName;
			IsolatedStorageFile isoFile = GetStore();
			string[] files = isoFile.GetFileNames(fileName);
			if (files.Length == 1) {
				// file exists -> Deserialize
				using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(
						   fileName, FileMode.Open, isoFile)) {
					BinaryFormatter formatter = new BinaryFormatter();
					try {
						_settings = (Hashtable)formatter.Deserialize(stream);
					}
					catch (SerializationException) {
						_settings = new Hashtable();
					}
				}
			} else {
				// start fresh
				_settings = new Hashtable();
			}
		}

		/// <summary>
		/// ND Destructor
		/// </summary>
		~IsolatedUserSettings() {
			Dispose(false);
		}

		/// <summary>
		/// Close the IsolatedUserSettings
		/// </summary>
		public void Dispose() {
			Dispose(true);
			System.GC.SuppressFinalize(this);
		}

		/// <summary>
		/// Close the IsolatedUserSettings
		/// </summary>
		/// <param key="disposing"></param>
		protected virtual void Dispose(bool disposing) {
			this.Close();
		}

		/// <summary>
		/// Write the UserSettings to the IsolatedStorage
		/// </summary>
		public virtual void Close() {
			if (!_closed) {
				IsolatedStorageFile isoFile = GetStore();

				using (IsolatedStorageFileStream stream = new IsolatedStorageFileStream(
						   FileName, FileMode.Create, isoFile)) {
					BinaryFormatter formatter = new BinaryFormatter();
					formatter.Serialize(stream, _settings);
				}
				_closed = true;
			}
		}

		/// <summary>
		/// Get the fileName
		/// </summary>
		public string FileName {
			get { return _fileName; }
		}

		/// <summary>
		/// Return the Store
		/// </summary>
		/// <returns></returns>
		private IsolatedStorageFile GetStore() {
			return IsolatedStorageFile.GetStore(_scope, null, null);
		}

		/// <summary>
		/// Register a Form 
		/// </summary>
		/// <param key="key"></param>
		/// <param key="form"></param>
		public virtual void RegisterForm(string key, Form form) {
			FormSetting f;
			if (_settings.ContainsKey(key)) {

				//f = (FormSetting)_settings[key];
				f = (FormSetting)GetSetting(key, new FormSetting(key));
				f.Restore(form);
			} else {
				f = new FormSetting(key);
				f.Backup(form);
				//_settings.Add(key, f);
				this.SaveSetting(key, f);
			}
			//form.Closed += new EventHandler(f.ClosedForm);
			form.SizeChanged += new EventHandler(f.FormMovedOrResized);
			form.LocationChanged += new EventHandler(f.FormMovedOrResized);
		}

		/// <summary>
		/// UnRegister a Form 
		/// </summary>
		/// <param key="key">Name of the Form. A forms with an existing key will 
		/// automatically get the stored settings.</param>
		/// <param key="form"></param>
		public virtual void UnRegisterForm(string key, Form form) {
			if (_settings.ContainsKey(key)) {
				FormSetting f = (FormSetting)GetSetting(key);
				if (f != null) {
					f.Backup(form);
					//form.Closed -= new EventHandler(f.FormMovedOrResized);
					form.SizeChanged -= new EventHandler(f.FormMovedOrResized);
					form.LocationChanged -= new EventHandler(f.FormMovedOrResized);
				}
			}
		}

		/// <summary>
		/// Save a setting with a given key
		/// </summary>
		/// <param key="key">Name of the setting by which it is saved an can be 
		/// restored using GetSetting().</param>
		/// <param key="setting"></param>
		public void SaveSetting(string key, object setting) {

			// Check Serializable attribute
			TypeAttributes attributes = setting.GetType().Attributes;
			if ((attributes & TypeAttributes.Serializable) == 0) {
				throw new ArgumentException( "Setting argument must be marked Serializable", "setting");
			}

			// Add or Overwrite setting
			if (_settings.ContainsKey(key)) {
				_settings[key] = new SerializableSetting(key, setting);
			} else {
				_settings.Add(key, new SerializableSetting(key, setting));
			}
		}

		/// <summary>
		/// Return the setting with the given key
		/// </summary>
		/// <param key="key"></param>
		/// <returns>
		/// The setting object or null when the the given key does not exists
		/// </returns>
		public object GetSetting(string key) {
			return GetSetting(key, null);
		}

		/// <summary>
		/// Return the setting with the given key
		/// </summary>
		/// <param key="key"></param>
		/// <returns></returns>
		public object GetSetting(string key, object defaultValue) {
			if (_settings.ContainsKey(key)) {
				SerializableSetting o = ((SerializableSetting)_settings[key]);
				if (o.Data == null) {
					// Deserialization caused an Exception, use the defaultValue!
					o.Data = defaultValue;
					return defaultValue;
				}
				return o.Data;
			}
			return defaultValue;
		}
	}
}
